//////////////////////////////////
//
// Implementation of the CColour class
//
// Written for COMPSCI 372, May 2003. Richard Lobb.
// Largely just a copy of Burkhard's CVec3df class.
//
// A colour is a 3-tuple (R,G,B).
//
// A colour behaves just like a 3-vector except where multiplication is
// concerned: multiplication of two colours is componentwise and yields
// a colour. Also, the getGLColour methods return a four element array
// with the fourth element = 1 for compatability with OpenGL's RGBA model.
//
//////////////////////////////////

#include <iostream>
#include <math.h>
#include "Utilities.h"
#include "CColour.h"
using namespace std;

CColour::CColour() { // Default constructor yields black
    c[R] = 0.0; c[G] = 0.0; c[B] = 0.0;
}

CColour::CColour(float r, float g, float b) {
    c[R] = r; c[G] = g; c[B] = b; c[3] = 1;
}

CColour::CColour(const CColour& c1) { // Copy constructor
    c[R] = c1.c[R]; c[G] = c1.c[G]; c[B] = c1.c[B]; c[3] = 1;
}

CColour& CColour::operator=(const CColour& c1) {  // Assignment operator
    c[R] = c1.c[R]; c[G] = c1.c[G]; c[B] = c1.c[B]; c[3] = 1;
    return *this;
}

CColour& CColour::operator+=(const CColour& c1) { // += operator
    c[R] += c1.c[R]; c[G] += c1.c[G]; c[B] += c1.c[B];
    return *this;
}

CColour& CColour::operator-=(const CColour& c1) {
    c[R] -= c1.c[R]; c[G] -= c1.c[G]; c[B] -= c1.c[B];
    return *this;
}

CColour& CColour::operator*=(float scalar) {
    c[R] *= scalar; c[G] *= scalar; c[B] *= scalar;
    return *this;
}

CColour operator+(const CColour& c1, const CColour& c2) {
    return CColour(c1.c[R]+c2.c[R], c1.c[G]+c2.c[G], c1.c[B]+c2.c[B]);
}

CColour operator-(const CColour& c1, const CColour& c2) {
    return CColour(c1.c[R]-c2.c[R], c1.c[G]-c2.c[G], c1.c[B]-c2.c[B]);
}

CColour operator*(float scalar, const CColour& c1) {
    CColour c0 = c1;
    c0 *= scalar;
    return c0;
}

CColour operator*(const CColour& c1, float scalar) {
    CColour c0 = c1;
    c0 *= scalar;
    return c0;
}

CColour operator*(const CColour& c1, const CColour& c2) {
    return CColour(c1.c[R]*c2.c[R], c1.c[G]*c2.c[G], c1.c[B]*c2.c[B]);
}

CColour operator-(const CColour& c1) {
    return CColour(-c1.c[R], -c1.c[G], -c1.c[B]);
}

bool operator==(const CColour& c1, const CColour& c2) {
    return (c1.c[R]==c2.c[R] && c1.c[G]==c2.c[G] && c1.c[B]==c2.c[B]);
}

bool operator!=(const CColour& c1, const CColour& c2)
{
    return (c1.c[R]!=c2.c[R] || c1.c[G]!=c2.c[G] || c1.c[B]!=c2.c[B] );
}

ostream& operator<<(ostream& s, const CColour& c1)
{
    return s << '(' << c1.c[R] << ", " << c1.c[G] << ", " << c1.c[B] << ')';
}

istream& operator>>(istream& s, CColour& c1)
{
    char c = 0;

    s >> c;
    if (c != '(')
        displayMessage(ERROR_MESSAGE, "CColour::>>. bad input format");
    s >> c1.c[R];
    s >> c;
    if (c != ',')
        displayMessage(ERROR_MESSAGE, "CColour::>>. bad input format");
    s >> c1.c[G];
    s >> c;
    if (c != ',')
        displayMessage(ERROR_MESSAGE, "CColour::>>. bad input format");
    s >> c1.c[B];
    s >> c;
    c1.c[3] = 1;
    if (c != ')')
        displayMessage(ERROR_MESSAGE, "CColour::>>. bad input format");
    return s;
}
