//////////////////////////////////
//
// Header file for the CColour class
//
// Written for COMPSCI 372, May 2003. Richard Lobb.
// Largely just a copy of Burkhard's CVec3df class.
//
// A colour is a 3-tuple (R,G,B).
//
// A colour behaves just like a 3-vector except where multiplication is
// concerned: multiplication of two colours is componentwise and yields
// a colour. Also, the getGLColour methods return a four element array
// with the fourth element = 1 for compatability with OpenGL's RGBA model.
//
//////////////////////////////////

#if !defined(CCOLOUR_372FC)
#define CCOLOUR_372FC

#include <iostream>

typedef enum { R, G, B } Components;

class CColour {
	public:
	    CColour();
	    CColour(float r, float g, float b);
		  CColour(const CColour& c); // Copy constructor

	    // Assignment operator
	    CColour& operator=(const CColour& c1);

		  // Vector in array form
		  float* getGLColour() { return c;};
		  void getGLColour(float *array) { array[0]=c[0]; array[1]=c[1]; array[2]=c[2]; array[3]=c[3];}
		  void set(float r, float g, float b) { c[0] = r; c[1] = g; c[2] = b; c[3] = 1; }
		  void set(float *array) { c[0]=array[0]; c[1]=array[1]; c[2]=array[2]; c[3] = 1; }

	    // some other operators
	    float& operator[](int index) { return c[index]; }
	    CColour& operator+=(const CColour& c1);
	    CColour& operator-=(const CColour& c1);
	    CColour& operator*=(float scalar);
	    CColour& operator*=(CColour& c1);     // Componentwise multiply
	    CColour& operator/=(float scalar);

	    friend CColour operator+(const CColour& c1, const CColour& c2);
	    friend CColour operator-(const CColour& c1, const CColour& c2);
	    friend CColour operator*(float scalar, const CColour& c1);
	    friend CColour operator*(const CColour& c1, float scalar);
	    friend CColour operator*(const CColour& c1, const CColour& c2);  // Componentwise
	    friend CColour operator-(const CColour& c1);
	    friend bool operator==(const CColour& v1, const CColour& c2);
	    friend bool operator!=(const CColour& v1, const CColour& c2);

	    // I/O operator
	    friend ostream& operator<<(ostream& s, const CColour& c1);
	    friend istream& operator>>(istream& s, CColour& c1);

	private:
	   float c[4];  // The colour, stored as a 4-tuple
};

#endif // !defined(CCOLOUR_372FC)
