// CRayTraceCamera.h: interface for the CRayTraceCamera class.
// A class that makes ray-traced images of a scene to a GLUT window.
//
// Its rayTrace method must be called from within the OpenGL display function.
//
// Written for COMPSCI 372, May 2003, Richard Lobb.
//////////////////////////////////////////////////////////////////////

#if !defined(RAY_TRACE_CAMERA_372FC)
#define RAY_TRACE_CAMERA_372FC
#include "geometry.h"
#include "SceneDescription.h"

const static float PI = 3.1415926f;        // An old friend

class CRayTraceCamera {
    private:
        CVec3df eyePos;         // Where the eye (aka camera) is
        CVec3df u, v, n;        // The eye coordinate system vectors
        float fov;              // Field Of View (degrees)
    public:
        CRayTraceCamera(const CVec3df& eyePosition, const CVec3df& lookat,
                        const float fovDegs);
        void rayTrace(CScene& scene);
             // Ray traces the given scene to an existing GLUT window
             // Must be called from within "display" function.
        friend ostream& operator<<(ostream& s, CRayTraceCamera& cam);
            // Define "<<" however you like (if you like!)
    private:
        void clearStats();  // Clear all the statistics-gathering variables
};

#endif // !defined(RAY_TRACE_CAMERA_372FC)
