// Transformation.h -- the header file for the affine transformation class.

// Based on Burkhard's CMatrix3df class, but is a 4 x 4 matrix in order
// to handle translations.

// Written for COMPSCI 372, Richard Lobb, May 2003.

#if !defined(CTRANSFORMATION_372FC)
#define CTRANSFORMATION_372FC

#include "Geometry.h"
#include <iostream>
class CTransformation {
	public:
	    // Constructors/ Destructor
	    CTransformation();
	    CTransformation(float m[4][4]);
	    CTransformation(const CTransformation& m1); // Copy constructor

	    // Assignment operator
	    CTransformation& operator=(const CTransformation& m1);

	    // some other operators
	    float& operator()(int i, int j) { return m[i][j]; }
	    CTransformation& operator*=(const CTransformation& m1);
	    CTransformation& operator*=(float scalar);
	    CTransformation& operator/=(float scalar);
	    friend CTransformation operator*(float scalar, const CTransformation& m1);
	    friend CTransformation operator*(const CTransformation& m1, float scalar);
	    friend CTransformation operator/(const CTransformation& m1, float scalar);
	    friend CVec3df operator*(const CTransformation& m1, const CVec3df& v1);
	    friend CVec3df operator*(const CVec3df& v1, const CTransformation& m1);
	    friend CTransformation operator*(const CTransformation& m1, const CTransformation& m2);

	    // I/O operator
	    friend ostream& operator<<(ostream& s, const CTransformation& m1);
	    friend istream& operator>>(istream& s, CTransformation& m1);

        // Transformation of a VECTOR!! This differs from the usual "*"
        // operator, which is used to transform POINTS (albeit represented
        // by position vectors of type CVec3df). This method assumes that the
        // 4th coordinate of the given CVec3df in homogeneous coordinates is 0.
        CVec3df transformVector(CVec3df& v);

	    // calculation of the inverse transformation
	    CTransformation inverse(void) const;

        // Calculation of the transpose
        CTransformation transpose(void) const;

	protected:
		float m[4][4];	// transformation stored in a static 4 x 4 array.
};

// Now some subclasses, defined in order to provide nicely named constructors
// for various common transformations.

class CTranslate : public CTransformation { // A translation
    public:
       CTranslate(CVec3df& shift);
       CTranslate(float dx, float dy, float dz);
};

class CScale : public CTransformation {  // A scaling
    public:
        CScale(float xScale, float yScale, float zScale);
};

class CRotate : public CTransformation {  // A rotation
    public:
        CRotate(int axis, float angleInDegs); // Axis = 0, 1, 2 for x, y, z.
        CRotate(CVec3df& axis, float angleInDegs);
};

class CIdentity : public CTransformation { // The identity matrix
    public:
        CIdentity();
};

#endif // CTRANSFORMATION_372FC
