// The main program for the 372 Ray Tracer assignment, 2003.
//
// This program is essentially a simplified version of
// Hill's ray tracer, except that I've modified the scene
// description classes somewhat. Also, it uses Burkhard's
// geometry classes and my own transformation class rather
// Hill's.

// Richard Lobb, April 2003.

// Further modified for use with BSP tree subdivision for
// 715, August 2003.

#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glut.h>
#include "Geometry.h"
#include "RayStuff.h"
#include "SceneDescription.h"
#include "CRayTraceCamera.h"
#include <iostream>

CVec3df up(0,1,0);

//CVec3df eye(0,0, 5);
//CVec3df eye(0.1f,0.1f,5);

CVec3df eye(4,8,12);
CVec3df lookat(-10,0,-50);

float fovDegs = 35;
CRayTraceCamera cam(eye, lookat, fovDegs);
CScene* scene;

void display() {
     glClear(GL_COLOR_BUFFER_BIT );
     cam.rayTrace(*scene);
     glFlush();
}

// Define the prototype for the scene building function.
//CScene* sphereAndCylinderScene(int gridSize, int maxNumObjInLeaf, int maxTreeDepth);
CScene* fenceScene(int numElements, int maxNumObjInLeaf, int maxTreeDepth);

int main(int argc, char* argv[]) {  // The main program.
     // Usage is: RayTracer numFenceElements maxNumObjInLeaf maxTreeDepth
	 // (but if no parameters, RayTracer 500 5 20)
     
     // where:
     //     numFenceElements is the number of fence elements in the scene
     //     maxNumObjInLeaf is the maximum number of objects that the
     //          BSP tree algorithm should allow in a BSP leaf node
     //          (but only if maxTreeDepth is not exceeded)
     //     maxTreeDepth is the maximum allowable depth of the BSP tree.
     
     
     if (argc != 4 && argc != 1) {
          cout << "Usage: RayTracer numFenceElements maxNumObjInLeaf maxTreeDepth\n";
          return -1;
     }
	 int numFenceElements =  argc > 1 ? atoi(argv[1]) : 500;    // The number of fence elements
	 int maxNumObjInLeaf = argc > 1 ? atoi(argv[2]) : 5;        // Maximum capacity of BSP leaf
	 int maxTreeDepth = argc > 1 ? atoi(argv[3]) : 20;          // Maximum depth of BSP tree

	 scene = fenceScene(numFenceElements, maxNumObjInLeaf, maxTreeDepth);
	 // scene = sphereAndCylinderScene(gridSize, maxNumObjInLeaf, maxTreeDepth);

	 // Now set up for rendering.

     glutInit(&argc, argv);
     glutInitDisplayMode(GLUT_RGB);
     glutInitWindowSize(300,200);
     glutInitWindowPosition(100,100);
     glutCreateWindow("715 Ray Tracer");
     glClearColor(1,1,1,1);
     glColor3f(0,0,0);
     glutDisplayFunc(display);
     glutMainLoop();
     return(0);
}
   
