// The function to build a scene made up of an n x n x n array of spheres
// interconnected by cylinders.

// Written for COMPSCI 715 2003, August 2003.  R J Lobb.

#include "Geometry.h"
#include "SceneDescription.h"

CColour lightAmbient(0.3f, 0.3f, 0.3f);
CColour lightDiffuse(1, 1, 1);
CColour lightSpecular(1,1,1);
CVec3df lightPos(0.1f, 0.1f, 5.0f);
CColour sphereMatDiffuse(0.3f, 0.3f, 0.3f);
CColour sphereMatSpecular(0.3f, 0.3f, 0.3f);
CColour black(0, 0, 0);
CMaterial sphereMat(sphereMatDiffuse, sphereMatDiffuse, sphereMatSpecular, 200, 0.6f, 0);

CColour cylMatDiffuse(0.7f, 0.3f, 0.2f);
CColour cylMatSpecular(0.5f, 0.5f, 0.5f);
CMaterial cylMat( cylMatDiffuse, cylMatDiffuse, cylMatSpecular, 20);

CScene* sphereAndCylinderScene(int gridSize, int maxNumObjInLeaf, int maxTreeDepth) {
     
     const float SIZE = 3.0f/(5*gridSize - 3); // Radius of each sphere
     const float CSIZE = SIZE/3;               // Cylinder radius
     float delta = 2.0f / (gridSize - 1);      // Spacing of spheres
     
     CSceneObject* pScene = NULL;  // Pointer to the first object in the scene
     CSceneObject* pLast = NULL;   // Pointer to the last object in the scene
     
     // The next two arrays define the required rotation of the cylinders
     // connecting the spheres -- a rotation of "rotAngle[axis]" around axis
     // rotAxis[axis] is required to align the cylinder with the given axis.
     
     int rotAxis[] = {1,0,2};
     float rotAngle[] = {90.0f, -90.0f, 0.0f};
     
     // Loop over all three dimensions to put a sphere and associated
     // cylinders at each grid point.
     
     for (int nx = 0; nx < gridSize; nx++)
          for (int ny = 0; ny < gridSize; ny++)
              for (int nz = 0; nz < gridSize; nz++) {
                  CVec3df centre(-1 + nx * delta, -1 + ny * delta, -1 + nz * delta);
                  CTransformation tx = CTranslate(centre) * CScale(SIZE, SIZE, SIZE);
                  CSceneObject* pObj = new CSphere(tx, sphereMat);
                  if (pScene == NULL) {
                      pScene = pObj;
                      pLast = pObj;
                  }
                  else {
                      pLast->next = pObj;
                      pLast = pObj;
                  }
                  for (int axis = 0; axis < 3; axis++) {  // Add the connecting cylinders
                      if (centre[axis] < 0.99999f) {
                          CTransformation tx = CTranslate(centre) * CRotate(rotAxis[axis],rotAngle[axis]) * CScale(CSIZE, CSIZE, delta);
                          CSceneObject* pCyl = new CTaperedCylinder(1, tx, cylMat);
                          pLast->next = pCyl;
                          pLast = pCyl;
                      }
                  }                      
              }

     CLight* lights = new CLight[1];
	 lights[0] = CLight(lightPos, lightAmbient, lightDiffuse, lightSpecular);
	 float w = 1.1f + SIZE;  // Scene halfwidth
     CBBox3f bbox(-w, w, -w, w, -w, w);
     
     // Finally, assemble scene objects and lights into a CScene object.
     // The BSP tree is built in the constructor.
     
     CScene* scene = new CScene(pScene, bbox, 1, lights, maxNumObjInLeaf, maxTreeDepth);
	 return scene;
}