// The function to build a scene made up of a sequence of rectangles
// arrayed along the -ve z axis as a test case for antialiasing algorithms.
// There's also a ground polygon on which the rectangles cast a shadow,
// to make it a little less boring.

// Written for COMPSCI 715 2003, August 2003.  R J Lobb.

#include "Geometry.h"
#include "SceneDescription.h"

CColour lightAmbient(0.3f, 0.3f, 0.3f);
CColour lightDiffuse(1, 1, 1);
CColour lightSpecular(1,1,1);
CVec3df lightPos(1000000, 1000000, 0);
CColour fenceMatDiffuse(0.7f, 0.6f, 0.2f);
CColour fenceAmbient = 0.3f * fenceMatDiffuse;
CColour fenceMatSpecular(0.3f, 0.3f, 0.3f);
CColour groundMatDiffuse(0.1f, 0.5f, 0.2f);
CColour groundAmbient = 0.3f * groundMatDiffuse;
CColour groundMatSpecular(0,0,0);
CMaterial fenceMat(fenceAmbient, fenceMatDiffuse, fenceMatSpecular, 50);
CMaterial groundMat(groundAmbient, groundMatDiffuse, groundMatSpecular, 0);

const float FENCE_ELEMENT_HEIGHT = 10.0f;

CScene* fenceScene(int numElements, int maxNumObjInLeaf, int maxTreeDepth) {
     
     CSceneObject* pScene = NULL;  // Pointer to the first object in the scene
     CSceneObject* pLast = NULL;   // Pointer to the last object in the scene

	 // Loop to construct "fence" of "numElements" rectangles.
	 // Rectangles are arrayed in planar fashion along the -ve z axis.
	 // Each rectangle is 1 unit wide and 10 units high and is sitting on the y = 0 plane.
	 // The first rectangle has its bottom left corner at (0,0).
	 // Rectangles are spaced 2 units apart (so there is a 1 unit gap between them).
          
	 for (int i = 0; i < numElements; i++) { 
	    CVec3df centre(0, FENCE_ELEMENT_HEIGHT/2, -2 * i - 0.5f);
		CTransformation tx = CTranslate(centre) * CRotate(1, 90) * CScale(1, FENCE_ELEMENT_HEIGHT, 1.0f);
		CSceneObject* pObj = new CSquare(tx, fenceMat);
		if (pScene == NULL) {
			pScene = pObj;
			pLast = pObj;
		}
		else {
			pLast->next = pObj;
			pLast = pObj;
		}
	 }

	 float farMost = 2 * numElements - 1.0f;	// depth of farthest edge of farthest fence elements
	 CVec3df groundCentre(0, 0, -farMost/2);
	 CTransformation txGnd = CTranslate(groundCentre) * CRotate(0,-90) * CScale(FENCE_ELEMENT_HEIGHT *2 , farMost, 1);
	 CSceneObject* pObj = new CSquare(txGnd, groundMat);
	 pLast->next = pObj;

     CLight* lights = new CLight[1];
	 lights[0] = CLight(lightPos, lightAmbient, lightDiffuse, lightSpecular);
     
     // Finally, assemble scene objects and lights into a CScene object.
     // The BSP tree is built in the constructor.
     
     CScene* scene = new CScene(pScene, 1, lights, maxNumObjInLeaf, maxTreeDepth);
	 return scene;
}
